@@ -63,7 +63,7 @@ attributes(Mod) ->
     try
         [?attribute(Key, Val) || {Key, Val} <-
             proplists:get_value(attributes, Mod:module_info(), []),
-            Key =/= vsn, Key =/= deprecated]
+            Key =/= vsn, Key =/= deprecated, Key =/= optional_callbacks]
     catch
         error:undef -> []
     end.
@@ -144,7 +144,7 @@ var_name(A) -> list_to_atom("A"++integer_to_list(A)).
 -spec exec(CallerPid::pid(), Mod::atom(), Func::atom(), Args::[any()]) ->
         Result::any().
 exec(Pid, Mod, Func, Args) ->
-    case meck_proc:get_result_spec(Mod, Func, Args) of
+    try meck_proc:get_result_spec(Mod, Func, Args) of
         undefined ->
             meck_proc:invalidate(Mod),
             raise(Pid, Mod, Func, Args, error, function_clause);
@@ -160,6 +160,9 @@ exec(Pid, Mod, Func, Args) ->
             after
                 erase(?CURRENT_CALL)
             end
+    catch
+        error:{not_mocked, Mod} ->
+            apply(Mod, Func, Args)
     end.
 
 -spec handle_exception(CallerPid::pid(), Mod::atom(), Func::atom(),
